# HABSKY1 -- Divide extracted multispec spectrum file by sky absorption spectra
#          saving original file in Noskyab/
# By Doug Mink, SAO Telescope Data Center
# May 22, 2007

procedure habsky1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
string	skyfile="hectospec$lib/skyabs.ms.fits"	{prompt="Sky absorption spectrum normalized to continuum=1"}
bool	deltemp = no	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save unprocessed file in Noskyab/ (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file newfile, imin, imsky, imoutf, imvar
	int lroot
	string imout, history, ap, keypath
	string bands	# List of input bands or lines/bands
	int npix, nap

	bands=""
	imin = image
	vb = verbose
	db = debug
	imsky = skyfile
	db = debug

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

	if (imin == "") {
	    print ("HABSKY1:  Divide extracted multispec spectrum file by sky absorption spectrum")
	    print ("Usage: habsky1 <input image>")
	    return
	    }

# Make sure input is a FITS multispec file
	smatch (imin, ".ms")
	if (!smatch.found) {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		}
	    else {
		imin = imin // ".ms.fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	if (!access (imin)) {
	    print ("HABSKY1: ", imin," does not exist")
	    return
	    }

# Do not remove sky absorption twice
	imgetp.default = "0"
	imgetp (imin, "HABSKY")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HABSKY: ", imin, " has already been processed")
		print ("HABSKY = ", imgetp.value)
		}
	    return
	    }

# Copy original file to Noskyab directory
	if (saveold) {
	    if (!access ("Noskyab/")) {
		mkdir ("Noskyab")
		}
	    newfile = "Noskyab/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    imcopy.verbose = db
	    imcopy (imin, "Noskyab")
	    if (vb) {
		print ("HABSKY1: Original ", imin, " => Noskyab/")
		}
	    }


# Get number of apertures and pixels from header of input multispec image
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	if (vb) {
	    print ("HABSKY1: ", nap," apertures of ",npix," pixels each")
	    }

# Divide out normalization spectra
	imout = mktemp ("abskyout") // ".ms.fits"
	imoutf = mktemp ("abskyoutf") // ".ms.fits"
	sarith.w1 = INDEF
	sarith.w2 = INDEF
	sarith.apertures = "1-" // nap
	sarith.bands = bands
	sarith.beams = ""
	sarith.apmodulus = 0
	sarith.reverse = no
	sarith.ignoreaps = no
	sarith.format = "multispec"
	sarith.renumber = no
	sarith.offset = 0
	sarith.clobber = yes
	sarith.merge = no
	sarith.rebin = yes
	sarith.errval = 0.
	sarith.verbose = db
	sarith (imin, "/", imsky, imout)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " " // imsky // " divided out"
	hedit (imout, "HABSKY", history, add+, verify-, show-)
	if (vb) {
	    print ("HABSKY: ",history)
	    }

# Rename output file to same as input file (which is saved if saveold=yes)
	imdelete (imin)
	imcopy (imout, imin)

	if (deltemp) {
	    imdelete (imout)
	    }

# Divide variance spectrum by throughput correction twice, if it exists
	imroot (imin, verbose=db)
	imvar = imroot.root // "_v.ms.fits"
	if (access (imvar)) {

# Do not remove sky absorption vrom variance spectrum if already done
	    imgetp.default = "0"
	    imgetp (imin, "HABSKY")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HABSKY: ", imvar, " has already been processed")
		    print ("HABSKY = ", imgetp.value)
		    }
		return
		}

# Remove sky absorption twice from variance
	    imtemp = mktemp ("vartemp")
	    sarith (imvar, "/", imsky, imtemp, verbose=yes)
	    if (saveold) {
		newfile = "Noskyab/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (imvar, "Noskyab")
		}
	    sarith (imtemp, "/", imsky, imvar, verbose=yes)
	    hedit (imvar, "HABSKY", history, add+, verify-, show-)
	    }
end

# Aug  6 2004	New program
# Aug 26 2004	Make it work on 1D files, too
# Aug 27 2004	Change logging keyword to HABSKY from SKYAB
# Aug 31 2004	Don't process image if HABSKY keyword is in header
# Aug 31 2004	Read next image if there is a problem instead of stopping
# Sep  2 2004	Process multiple input images
# Oct 13 2004	Check for .ms, not .ms.fit
# Oct 14 2004	Write sarith output to new file and rename it
# Oct 14 2004	Replace sarith-written APNUMnnn with APIDnnn from input file
# Oct 18 2004	Print HABSKY value if already processed

# Jan 11 2005	Improve input filename options
# Feb 23 2005	Drop keyword copying; use multispec version of sky spectrum
# Apr 14 2005	Really delete temporary files if requested
# Apr 15 2005	Fix APID keywords if they are deleted by IRAF
# May 31 2005	Make into single file task HABSKY1 to call from HTRAN
# Jun 30 2005	Change external cphead,delhead calls to wcstools.chead,dhead

# Feb 13 2007	Set sarith.ignoreaps=no to preserve APIDi keyword values
# Feb 15 2007	Add variance spectrum computation
# May  7 2007	Find image root name using imroot()
# May 22 2007	Only check for wcstools package, not dhead task in it
# May 24 2007	Undeclare imroot; add verbose=db to imroot call
