# HEXTRACT -- Extract  Hectospec image into multispec file
# By Doug Mink, SAO Telescope Data Center
# November 14, 2006

procedure hextract (images)

file	images=""	{prompt="Input image(s) to extract"}
file	mapfile=""	{prompt="Fiber to sky map file"}

file	mapdir=""	{prompt="Directory for mapfiles"}
bool	mapfix=no	{prompt="Replace tabs and fix names in map file"}
file	apref=""	{prompt="Aperture reference spectrum"}
int	fibers=300	{prompt="Number of fibers"}
real	width=8.	{prompt="Width of profiles (pixels)"}
real	minsep=5.	{prompt="Minimum separation between fibers (pixels)"}
real	maxsep=100. 	{prompt="Maximum separation between fibers (pixels)"}
real	minshift=0.01	{prompt="Minimum shift to use in extraction (pixels)"}
string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
string	gain="1"	{prompt="Photon gain (photons/data number)"}
bool	resize=no	{prompt="Resize object apertures (y or n)"}
bool	recenter=no	{prompt="Recenter object apertures (y or n)"}
bool    shift=no	{prompt="Use average shift instead of recentering (y or n)"}
real    npeaks=0.2	{prompt="Select brightest peaks"}
bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print extra information for debugging (y or n)"}

begin
	bool vb, db
	file imin, imout, imroot, imref, imrefi, templist, mapd, map
	int lroot
	string history, ims, tchar
	real xshift, yshift

	ims = images
	imref = apref
	vb = verbose
	db = debug

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("extractlist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name
	    smatch (imin, ".ms")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}
	    imin = imroot // ".fits"
	    imout = imroot // ".ms.fits"

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "HEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HEXTRACT: ", imin, " is already an extracted image")
		    print ("HEXTRACT = ", imgetp.value)
		    }
		next
		}
	    if (db) {
		print ("HEXTRACT: ", imin, " -> ", imout)
		}

# Set up aperture template file with shifts from HSHIFT parameter in header
#	    if (shift) {
#		xshift = 0.0
#		}
#	    else {
#		imgetp.default = "0.0"
#		imgetp (imin, "HSHIFTX")
#		xshift = real (imgetp.value)
#		}
#	    imgetp.default = "0.0"
#	    imgetp (imin, "HSHIFTY")
#	    yshift = real (imgetp.value)
#	    if (db) {
#		printf ("HEXTRACT: shift from template of x= %.4f y=%.4f\n", xshift, yshift)
#		}
#	    if (xshift > minshift || xshift < -minshift ||
#		yshift > minshift || yshift < -minshift) {
#		imrefi = imref // "_" // imroot
#		haddshift (imref, imrefi, xshift=xshift, yshift=yshift, verbose=vb, debug=db)
#		if (vb) {
#		    printf ("HEXTRACT: template shifted by x= %.4f y=%.4f\n", xshift, yshift)
#		    }
#		}
#	    else {
		imrefi = imref
#		}

# Look for map file name in header
	    imgetp.default = "0"
	    imgetp (imin, "MAPFILE")
	    if (imgetp.value != "0") {
		map = imgetp.value
		}
	    else if (mapfile != "") {
		map = mapfile
		}

# If map file is not found or specified as a parameter, generate it
# There is ome map file per pointing, for multiple exposures=filenames
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    map = substr (imin,1,smatch.start-6) // "_map"
		    map = mapdir // map
		    if (!access (map)) {
			print ("HEXTRACT: Cannot find map file ", map)
			map = substr (imin,1,smatch.start-1) // "_map"
			map = mapdir // map
			if (!access (map)) {
			    print ("HEXTRACT: Cannot find map file ", map)
			    next
			    }
			}
		    }
		else {
		    smatch (imin, ".ms")
		    if (smatch.found) {
			map = substr (imin,1,smatch.start-6) // "_map"
			map = mapdir // map
			if (!access (map)) {
			    map = substr (imin,1,smatch.start-1) // "_map"
			    map = mapdir // map
			    if (!access (map)) {
				map = imin // "_map"
				map = mapdir // map
				if (!access (map)) {
				    print ("HEXTRACT: Cannot find map file ",map)
				    next
				    }
				}
			    }
			}
		    }
		}
	    map = mapdir // map
	    if (vb) {
		print ("HEXTRACT: ", imin," -> ",imout)
		print ("    using ", imrefi," and ",map)
		}

# Remove excess space from map file so its lines fit on header lines
# and fix object name so it will pass through IRAF intact
	    if (mapfix && map != "") {
		hsmapfix (map)
		}

# Set up dohspec parameters
	    dohspec.resize = resize
	    dohspec.apidtable = map
	    dohspec.apref = imrefi
	    dohspec.flat = ""
	    dohspec.throughput = ""
	    dohspec.arcs1 = ""
	    dohspec.arcs2 = ""
	    dohspec.arcreplace = ""
	    dohspec.arctable = ""
	    dohspec.readnoise = readnoise
	    dohspec.gain = gain
	    dohspec.datamax = INDEF
	    dohspec.fibers = fibers
	    dohspec.width = width
	    dohspec.minsep = minsep
	    dohspec.maxsep = maxsep
	    dohspec.crval = "INDEF"
	    dohspec.cdelt = "INDEF"
	    dohspec.objaps = ""
	    dohspec.skyaps = ""
	    dohspec.arcaps = ""
	    dohspec.recenter = recenter
	    dohspec.objbeams = "1"
	    dohspec.skybeams = "0"
	    dohspec.arcbeams = ""
	    dohspec.scattered = no
	    dohspec.fitflat = no
	    dohspec.clean = no
	    dohspec.dispcor = no
	    dohspec.savearcs = no
	    dohspec.skyalign = no
	    dohspec.skysubtract = no
	    dohspec.skyedit = no
	    dohspec.saveskys = no
	    if (vb || db) {
		dohspec.splot = yes
		}
	    else {
		dohspec.splot = no
		}
	    dohspec.redo = no
	    dohspec.update = no
	    dohspec.batch = no
	    dohspec.listonly = no
	    dohspec.shift = shift
	    dohspec.npeaks = npeaks
	    dohspec.params = ""

# Extract spectra to multispec file using dohspec
	    dohspec (imin)

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb
	    hedit (imout, "HEXTRACT", history, add+, verify-)
	    hedit (imout, "HEXAPREF", imrefi, add+, verify-)
	    hedit (imout, "HEXAPMAP", map, add+, verify-)
	    }

	if (deltemp && tchar != '@') {
	    delete (templist)
	    }

end

# Jun 15 2004	Add recenter, shift, npeaks, parameters for dohspec and proc
# Jun 16 2004	Add resize parameter for dohspec and apscript
# Sep  2 2004	Add HEXTRACT keyword when processed
# Oct 18 2004	Print HEXTRACT keyword if already processed

# Jan 12 2005	Set more dohspec parameters from hextract parameters
# Mar 21 2005	Image map name can be made from input file name .fits -> _map
# Mar 21 2005	Do not change deltemp if reading from file list
# Mar 21 2005	Read from list of .ms.fits output files or .fits input files
# Mar 24 2005	Change temporary file from "mergelist" to "extractlist"
# Mar 29 2005	WFW - turn off splot at end unless verbose or debug
# Apr  4 2005	DJM - combine changes made by Bill and Doug
# May 17 2005	DJM - Use HSHIFT shifts from header on aperture reference
# Jul  6 2005	DJM - Check for map files with exposure sequence in them

# May 10 2006	DJM - Add to hsmapfix comment
# Nov 14 2006	DJM - Add mapfix parameter
