procedure skyprocx ()

string spectra="\*.fits"	{prompt="Files to process"}
string skymethod="skyfit"	{prompt="Sky removal method"}
string prefix=""	{prompt="Prefix string for output file names"}
string destdir="../skysub/"	{prompt="Output dir for sky-subtracted files"}
string skysort="skydist" {prompt="Type of sky fiber sorting (skydist/aperture)"}
int nskies=0		{prompt="Max number of skies to coadd"}
real fiberdist=0.0	{prompt="Max Fiber distance to sky"}
real skycert=4.0	{prompt="Max R value for certified sky"}
real skystdlim=0.2	{prompt="Max std of difference from average sky"}
bool skysame=yes        {prompt="Use skys only from same chip"}
string skyline="OH8399"	{prompt="Sky line to use for scaling factor"}
bool usearea=no         {prompt="Use sky line area vs. height in scaling"}
real weight=0.0         {prompt="Sky scaling factor. If 0.0, then compute it"}
int mpjobs=4		{prompt="Max number of multiprocessor jobs in parallel"}
string logfile="skyproc.results" {prompt="Name of output summary file"}
file uparmdir="home$uparm"	{prompt="UPARM directory"}
bool deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool verbose=yes	{prompt="note progress"}
bool debug=no		{prompt="note progress in detail"}

begin
    bool vb, dt
    float fiber_x, fiber_y, fiber_radius, skyweight, fiberlim
    int aperture, chipnum, fiber
    string imagetyp, object, imf
    string arg0, arg

    dt = deltemp
    vb = verbose

# limit of non-parked fibers
    fiberlim = 306.0

    arg = arg // " -i " // files // " -d " // destdir // " -n " // nskies
    arg = arg // " -f " // fiberdist // " -r " // skycert // " -e " // skyline
    arg = arg // " -k " // skysort
    arg = arg // " -c " // skysame // " -m " // skymethod  
    arg = arg // " -s " // skystdlim // " -j " // mpjobs // " -l " // logfile
    arg = arg // " -a " // usearea // " -w " // weight
    arg = arg // " -u " // osfn (uparmdir)
    print ("skyprocksh ",arg)
    skyprocksh (arg)

# Make sure output directory is there
    if (!access (destdir)) {
	mkdir (destdir)
	}

    skyfit = 0
    if (skymethod == "skyfit") {
	skyfit = 1
	skyweight = 0.0
	}

# Set parameters for IRAF tasks used by SKYPROC
    hsktask.oldpars = mktemp ("oldpars")
    hsktask.deltemp = dt
    hsktask.verbose = vb
    hsktask (imarith)
    hsktask (imcombine)
    hsktask (imdelete)
    hsktask (imexpr)
    hsktask (imrename)
    hsktask (imstack)
    hsktask (imstatistics)
    hsktask (xcsao)
    hsktask (emsao)
    hsktask (contpars)

    if (vb) {
	printf ("\n%s rev %s start at $(date)\n\n", $(basename $0), $REV)
	printf ("  Sky removal method = %s\n", skymethod)
	printf ("  FiberRadiusLimit = %.1f\n", fiberlim)
	printf ("  Sky sorting type = %s\n", skysort)
	printf ("  Nskys = %d\n  FiberDist = %.1f\n  SkyCert  <= %.1f\n", nskys, fiberdis,t skycert)
	printf ("  SkyStdLim = %.2f\n" $skystdlim
	if (skysame)
	    printf ("  Skys on same chip\n")
	else
	    printf ("  Skys on either chip\n")
	if (skymethod == "skysub) {
	    printf ("  Sky line used for scale: %s\n", emline)
	if 
	if (abs(skyweight) > 0.001) 
	    printf ("  Sky scaling: scale fixed at %.1f\n", skyweight)
	else if (usearea > 0)
	    printf "  Sky scaling: scale by line area\n")
	else
	    printf "  Sky scaling: scale by line height\n")
	printf ("  DB Files prefix = %s\n", prefix)
	printf ("  DestDir = %s\n\n", destdir)
	}

#  Make temporary list of input spectrum files
        tchar = substr (spectra,1,1)
        if (tchar == '@') {
            templist = substr (spectra,2,100)
            }
        else {
            templist = mktemp ("skylist")
            files (spectra, >templist)
            }

# Generate master table of keywords
	arg0 = "-au imagetyp object aperture fiber fiber_x fiber_y "
	tempkey = mktemp ("skykey")
        list = templist
	while (fscan (list, file) != EOF) {
	    gethead (arg, >>tempkey)
	    }

# Edit master keyword table
	nsky = 0
	nsky1 = 0
	nsky2 = 0
	nobj = 0
	list = tempkey
	while (fscan (list, imf, imagetype, object, aperture, fiber, fiber_x, fiber_y) != EOF) {
	    if (object == "-1") {
		object = "UNUSED"
		imagetyp = "UNUSED"
		}
	    else if (object == "-2") {
		object = "BROKEN"
		imagetyp = "BROKEN"
		}
	    else if (object == "0") {
		object = "SKY"
		}
	    fiber_radius = sqrt (fiber_x * fiber_x + fiber_y * fiber_y)
	    if (skysame) {
		if (aperture <= 150) {
		    chipnum = 1
		    }
		else {
		    chipnum = 2
		    }
		}
	    else {
		chipnum = 0
		}

# Pull all object and not sky files into a master list
	    if ((imagetyp == "object" || imagetyp == "OBJECT)) &&
		object != "UNUSED" && object != "BROKEN" &&
		object != "PARKED && fiber_radius <= fiberlim) {
		if (nobj < 0) {
		    printf ("hdrfile 	object  	chip	ap	fib	fiber_x	fiber_y\n",>objdb)
		    printf ("--------	--------	----	---	---	-------	-------\n",>objdb)
		    }
		printf ("%s	%s	%1d	%3d	%3d	%f	%f\n",
			imf,object,chipnum,aperture,fiber,fiber_x,fiber_y,>>objdb)
		nobj = nobj + 1
		}

# create a sky file db
	    if ((imagetyp == "OBJECT" && object == "SKY" && 
		fiber_radius <= fiberlim) {
		if (nsky < 1) {
		    printf ("hdrfile 	chip	ap	fib	fiber_x	fiber_y\n",>objdb)
		    printf ("--------	----	---	---	-------	-------\n",>objdb)
		    }
		printf ("%s	%1d	%3d	0.0\n",
			imf,chipnum,aperture,>>skyavstd.db)
		if (skysame) {
		    if (chipnum == 1) {
			print (imf,>>sky1list)
			nsky1 = nsky1 + 1
			}
		    else {
			print (imf,>>sky2list)
			nsky2 = nsky2 + 1
			}
		    }
		else
		    print (imf,>>skylist)
		nsky = nsky + 1
		}
	    }

# Return if no sky spectra to subtract
	if (nsky <= 0) {
	    print ("No sky spectra in this directory.")
	    exit -1
	    }

# compute RMS of individual skys against the sum of them; test for rejection
	if (skysame) {
	    avout = "skyav.1.fits"
	    del (avout)
	    imcombine (@sky1list,$avout,rejmask="",combine="average",
		       reject="avsigclip", outtype="real", scale="none",
		       zero="none",weight="!EXPTIME", mclip=yes, lsigma=3.,
		       hsigma=3., sigscale=0.1,logfile="")
	    avout = "skyav.2.fits"
	    del (avout)
	    imcombine (@sky2list,$avout,rejmask="",combine="average",
		       reject="avsigclip", outtype="real", scale="none",
		       zero="none",weight="!EXPTIME", mclip=yes, lsigma=3.,
		       hsigma=3., sigscale=0.1,logfile="")
	    }
	else {
	    avout = skyav.fits
	    del (avout)

	# compute master sky average, using 'valid' skys
	    imcombine (@skylist,$avout,rejmask="",combine="average",
		       reject="avsigclip", outtype="real", scale="none",
		       zero="none",weight="!EXPTIME", mclip=yes, lsigma=3.,
		       hsigma=3., sigscale=0.1,logfile="")

	    }

# Reset parameters to original values
    cl (<oldpars)
    if (dt)
	delete (oldpars)
end
